;;; www-mde.el
;;; Michael Ernst <mernst@research.microsoft.com>
;;; Last modified 3/95

;;; Contents:
;; Wordtext-to-html: extra-html.
;; Menu bars: insert-menu-bar[s], update-menu-bar[s].
;; Utilities

;; also see www-index.el

(require 'www-index)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; For each file from Macintosh, with no paragraph breaks:
;;  1. mactext-convert
;;  2. extra-html
;; update-menu-bars  (this does the whole directory)

;; To do:
;;  Sidebars (hard to do automatically, I would think)
;;  Deal correctly with Sound Bites in table of contents
;;  Make L.A. Times be treated as a whole index term from the beginning.
;;  Make saving much faster (it's hideously slow now).

(require 'util-mde)			; for current-buffer-line

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Wordtext-to-html
;;;

;; This is tuned for articles from EXTRA!.

(fset 'extra-html 'wordtext-to-html)

(defun wordtext-to-html ()
  (interactive)
  (if (string-match "\\.txt$" buffer-file-name)
      (progn
	(setq buffer-file-name (concat (substring buffer-file-name 0 -4) ".html"))
	(rename-buffer (concat (substring (buffer-name) 0 -4) ".html"))))

  ;; Gubbish at end of file and end of lines
  ;; (delete-blank-lines) doesn't get ^M characters
  (goto-char (point-max))
  (skip-chars-backward "\r\n\t \C-z")
  (kill-region (point) (point-max))
  (if (looking-back-at "\\?\\?")
      (backward-delete-char 2)
    (insert "\n"))
  (remove-trailing-whitespace)

  ;; Fix title if broken across lines
  (goto-char (point-min))
  ;; If separated by newlines
  (if (and (re-search-forward ".\nby " 200 t)
	   (= (current-buffer-line) 3))
      (progn
	(goto-char (point-min))
	(end-of-line)
	(delete-char 1)			; delete the newline
	(just-one-space)
	(goto-char (point-min))))
  ;; If separated by double newlines
  (if (and (re-search-forward "\n\nby " 200 t)
	   (= (current-buffer-line) 5))
      (progn
	(goto-char (point-min))
	(end-of-line)
	(delete-char 1)			; delete the newline
	(end-of-line)
	(delete-char 1)			; delete the newline
	(just-one-space)
	(goto-char (point-min))))

  ;; Make paragraphs be separated by <p>\n\n
  (goto-char (point-min))
  (let ((paragraph-sep (if (search-forward "\n\n" nil t) "\n\n" "\n")))
    (goto-char (point-min))
    ;; (replace-string paragraph-sep "<p>\n\n")
    (while (search-forward paragraph-sep nil t)
      (replace-match "<p>\n\n")))
  (goto-char (point-max))
  (skip-chars-backward "\n")
  (if (not (looking-back-at "<p>"))
      (insert "<p>"))

  ;; Maybe move Media Beat date
  (goto-char (point-min))
  (if (looking-at (concat month-regexp " [1-3]?[0-9], 199[0-9]$"))
      (let ((date (buffer-substring (point) (progn (end-of-line 1) (point)))))
	(delete-region (point-min) (progn (forward-line 1) (point)))
	(if (or (search-forward "By Jeff Cohen and Norman Solomon<p>" nil t)
		(and (or (search-forward "\n\n     " nil t)
			 (search-forward "\n\n"))
		     (progn (goto-char (match-beginning 0)) t)))
	    (insert "\n\n<a href=\"../#media-beat\">Media Beat</a>, " date "<p>"))))

  ;; Title (at beginning of article)
  (goto-char (point-min))
  (insert "<title>")
  (end-of-line)
  (if (not (looking-back-at "<p>"))
      (error "What happened to the <p>?"))
  (backward-delete-char 3)
  (insert "</title>\n<link rev=\"made\" href=\"mailto:mernst@theory.lcs.mit.edu\">\n\n<h1>" (buffer-substring (+ (point-min) 7) (point)) "</h1>")
  ;; Issue in which it appeared
  ;; This search should never fail, but just in case...
  (if (re-search-forward
       (concat "\n\nEXTRA! ?\\(Update\\)? \(\\|" month-regexp "/") nil t)
      (progn
	(beginning-of-line)
	(insert "<a href=\"toc.html\">")
	(end-of-line)
	(if (looking-back-at "<p>")
	    (goto-char (match-beginning 0)))
	(insert "</a>")))
  (forward-line 2)

  ;; Operations on body of article.
  (let ((article-begin (point)))
    (goto-char article-begin)
    (remove-post-slash-spaces)
    (goto-char article-begin)
    (query-replace-hyphen-with-em-dash)
    (let ((case-fold-search nil))
      (goto-char article-begin)
      (replace-string "ITAL>" "<em>")
      (goto-char article-begin)
      (replace-string "<ITAL" "</em>"))
    (goto-char article-begin)
    (fill-region (+ (point) 2) (point-max))
    (goto-char article-begin)
    (query-replace-regexp "\\([^-]\\)-[ \n]" "\\1-")
    (goto-char article-begin)
    (extra-make-subheaders)
    (goto-char article-begin)
    (while (search-forward "\nv " nil t)
      (replace-match "<UL>\n<li>")
      (while (progn (forward-paragraph)
		    (looking-at "\nv "))
	(replace-match "\n<li>")
	(backward-char 5)
	(if (not (looking-back-at "<p>\n"))
	    (error "Didn't find <p>\\n"))
	(replace-match "\n"))
      (if (not (looking-back-at "<p>\n"))
	  (error "Didn't find <p>\\n"))
      (replace-match "\n</UL>")))

  ;; Author information.
  (goto-char (point-min))
  (if (search-forward "\nby " 400 t)
      (let ((author (buffer-substring (point) (progn (end-of-line) (point)))))
	(goto-char (point-max))
	(backward-paragraph)
	(if (search-forward author nil t)
	    (progn
	      (backward-paragraph)
	      (insert "\n<hr>\n\n<address>")
	      (goto-char (point-max))
	      (if (looking-back-at "<p>\n?")
		  (progn
		    (goto-char (match-beginning 0))
		    (insert "</address>"))
		(insert "</address><p>"))))))

  (goto-char (point-min)))


(defvar hyphenated-words)
(setq hyphenated-words
      '("african-american" "african-americans" "award-winning"
	"bomb-throwing"
	"center-right" "cop-out"
	"dark-skinned"
	"flat-footed" "front-page" "full-page" "full-time"
	"heart-rending" "home-made"
	"inner-city" "iran-contra"
	"knee-jerk"
	"lock-step" "low-income"
	"media-beat" "middle-class"
	"name-calling" "neo-nazi"
	"on-line" "op-ed"
	"part-time"
	"rabble-rouser" "rip-off"
	"self-esteem" "secretary-general" "single-payer" "so-called"
	"vice-presidential"
	"well-documented" "white-collar" "wide-ranging" "working-class" "worn-out"
	"year-end"))

(defun query-replace-hyphen-with-em-dash ()
  ;; like (query-replace "-" " -- "), but makes some of the decisions for me
  (let ((begin (point)))
    (while (re-search-forward "-" nil t)
      (cond ((looking-at "-")
	     (forward-char 1))
	    ((looking-back-at " -")
	     (insert "- "))
	    ((looking-at "[a-z]+-")
	     ;; multiple hyphens in one work; forward-word won't skip them all
	     (forward-sexp 1))
	    ((looking-at "\\(and\\|but\\|which\\|such as\\|the\\|especially\\)[ \n]")
	     ;; not "and\\b" or "the\b"
	     (backward-delete-char 1)
	     (insert " -- "))
	    ((looking-back-at "\\<800-")
	     nil)
	    ((looking-back-at "\\(ex\\|re\\)-")
	     nil)
	    ((and (looking-back-at "-")
		  (or (looking-at "inch\\>")))
	     nil)
	    ((looking-back-at "\\<\\(anti\\|pro\\|non\\|co\\)-")
	     nil)
	    ((and (looking-back-at "mid-")
		  (looking-at month-regexp))
	     nil)
	    ((and (looking-back-at "\\<\\([0-9]+\\|one\\|two\\|three\\|four\\|five\\|six\\|seven\\|eight\\|nine\\|ten\\)-")
		  (or (looking-at "[1-9]")
		      (looking-at "\\(inch\\|time\\|part\\|page\\|sentence\\|thirds?\\|fourths?\\|fifths?\\|sixths?\\|second\\|minute\\|hour\\|day\\|week\\|month\\|year\\)\\>")))
	     nil)
	    ((and (looking-back-at "\\<\\(left\\|right\\)-") (looking-at "wing\\(er\\)?s?\\>"))
	     nil)
	    ((and (looking-back-at "\\<hard-") (looking-at "liner?s?\\>"))
	     nil)
	    ((member (save-excursion (downcase (buffer-substring
				      (progn (backward-word 1) (point))
				      (progn (forward-word 2) (point)))))
		     hyphenated-words)
	     nil)
	    ((message "Replace \"-\" with \" -- \"? ")
	     (let ((result (read-char)))
	       (if (memq result '(?y ?Y ?\ ))
		   (progn
		     (backward-delete-char 1)
		     (insert " -- ")))))))
    (goto-char begin)
    (query-replace "_" " -- ")))

(defun remove-post-slash-spaces ()
  (interactive)
  ;; First, special-case dates such as "10/ 24/94" and "3/14/ 67".
  (save-excursion
    (while (re-search-forward
	    "\\([0-9]?[0-9]/\\)\\s +\\([0-9]?[0-9]/[0-9][0-9]\\)" nil t)
      (replace-match "\\1\\2")))
  (save-excursion
    (while (re-search-forward
	    "\\([0-9]?[0-9]/[0-9]?[0-9]/\\)\\s +\\([0-9][0-9]\\)" nil t)
      (replace-match "\\1\\2")))
  (while (re-search-forward "\\w/\\(\\s +\\)" nil t)
    (message "Remove space after slash? ")
    (let ((result (read-char)))
      (if (memq result '(?y ?Y ?\ ))
	  (replace-match "")))))

(defun extra-make-subheaders ()
  (interactive)
  (if (string-match "sound-?bite" (buffer-name))
      (soundbites-make-subheaders)
    (if (interactive-p)
	(call-interactively 'non-soundbites-make-subheaders)
      (non-soundbites-make-subheaders))))

(defun soundbites-make-subheaders ()
  "Turn text at head of paragraph, preceding colon, into a header."
  (interactive)
  (goto-char (point-min))
  (while (re-search-forward "\n\n.*:" nil t)
    (delete-char 1)
    (backward-delete-char 1)
    (insert "</h2>\n\n")
    (backward-char 2)
    (beginning-of-line)
    (insert "<h2>")))

(defun non-soundbites-make-subheaders ()
  "Turn some one-line paragraphs into headers."
  (interactive)
  (if (interactive-p)
      (progn
	(goto-char (point-min))
	(search-forward "html\">EXTRA")))
  (while (re-search-forward "\n\n.*[^.?!]<p>\n\n" nil t)
    (forward-line -2)
    (message "Should this be a subheader?")
    (let ((result (read-char)))
      (if (memq result '(?y ?Y ?\ ))
	  (progn
	    (insert "<h2>")
	    (end-of-line)
	    (backward-delete-char 3)
	    (insert "</h2>"))))))

(defun extra-html-directory ()
  "Convert all text files in the current directory to HTML."
  (interactive)
  (let ((files (directory-files default-directory nil "\\.txt$")))
    (while files
      (find-file (car files))
      (extra-html)
      (setq files (cdr files)))))


(defun extra-make-toc ()
  (interactive)
  (let ((month-and-year (read-string "Month and year: "))
	(files (directory-files default-directory nil "\\.html$")))
    (kill-buffer "*extra-toc*")
    (set-buffer (get-buffer-create "*extra-toc*"))
    (insert
     "<title>" month-and-year
     "</title>\n<link rev=\\\"made\\\" href=\\\"http://www.research.microsoft.com/research/analysts/mernst/\\\">\n\n<h1>"
     month-and-year "</h1>\n\n"
     "<a href=\"cover-color.gif\"><img src=\"cover-color-small.gif\"></a><p>\n\n"
     "EXTRA! is the bimonthly magazine of <a href=\"../\">FAIR</a>.\n"
     "<a href=\"../#subscribe\">Subscribe</a> today!<p>\n\n"
     "Table of contents:\n<ul>\n")
    (while files
      (let ((file (car files)))
	(if (not (string= file "toc.html"))
	    (insert "  <li><a href=\"" file "\">"
		    (html-file->title file) "</a>\n")))
      (setq files (cdr files)))
    (insert
     "</ul>\n"
     "The printed versions of the magazine and newsletter may contain more\n"
     "material than the online versions.<p>\n\n"
     "<address>\n"
     "  <a href=\"http://www.research.microsoft.com/research/analysts/mernst/\">Michael Ernst</a>\n"
     "</address>\n"))
  (pop-to-buffer (current-buffer)))

(defun html-file->title (html-file)
  (let ((exists (get-file-buffer html-file)))
    (prog1
	(save-window-excursion
	  (save-excursion
	    (find-file html-file)
	    (save-excursion
	      (goto-char (point-min))
	      (re-search-forward "\\`<title>\\(.*\n?.*\\)</title>$")
	      (match-string 1))))
      (if (not exists) (kill-buffer (get-file-buffer html-file))))))

;; (html-file->title "~/WWW/fair/extra-9411/aristide-demonization.html")



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Macintosh text
;;;

(defun mactext-convert ()
  (interactive)
  (if (not (string-match "\\.txt$" buffer-file-name))
      (progn
	(setq buffer-file-name (concat buffer-file-name ".txt"))
	(rename-buffer (concat (buffer-name) ".txt"))))
  (goto-char (point-min))
  (skip-chars-forward " ")
  (delete-region (point-min) (point))
  (while (search-forward "\r  " nil t)
    (replace-match "\n" nil t))
  (goto-char (point-min))
  (remove-trailing-whitespace)
  (break-paragraphs)
  (goto-char (point-max))
  (skip-chars-backward "\r\n\t \C-z")
  (delete-region (point) (point-max))
  (if (looking-back-at "[0-9])")
      (let ((date (buffer-substring (1- (point))
				    (progn (backward-sexp) (1+ (point))))))
	(delete-region (point) (point-max))
	(goto-char (point-min))
	(if (search-forward "\n\nBy " 200 t)
	    (forward-line 1)
	  (progn
	    (re-search-forward "\n.+\n.+")
	    (forward-line -2)))
	;; Need to check whether this ought to be EXTRA!Update
	(insert "\nEXTRA! " date "\n")))
  (message "Possibly insert more paragraph breaks, then save as .txt."))

(defun break-paragraphs ()
  (interactive)
  (goto-char (point-min))
  (let ((case-fold-search nil))
    (while (not (eobp))
      (end-of-line 1)
      ;; I could check whether the next word would have fit on this line;
      ;; if it would have, then this is definitely a paragraph break.
      (if (or (< (current-column) 55)
	      (and (< (current-column) 70)
		   ;; end of sentence precedes
		   (looking-back-at "[.!?]\"?")
		   ;; uppercase letter follows
		   (looking-at "\n\"?[A-Z]")
		   (<= (+ (current-column) 1
			 (save-excursion (forward-word 1) (current-column)))
		       72)))
	  (insert "\n"))
      (forward-line 1))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Menu bars
;;;

;; Maybe the menu bar belongs at the top of some (non-article) pages too.

;; Eventual menu bar sections
;;  * next
;;  * up
;;  * top
;;  * comments
;;  * subscribe
;;  * search
;;  * index
;;  * show/hide index links (?)

(defun update-menu-bar ()
  (save-excursion
    (goto-char (point-min))
    (if (not (looking-at "<title>Page moved</title>"))
	(progn
	  (goto-char (point-max))
	  (search-backward "\n<hr>\n[ <a href=\"")
	  (let ((menu-bar-begin (1+ (point))))
	    (search-forward " ]<br>\n")
	    (delete-region menu-bar-begin (point)))
	  (insert-menu-bar)
	  ;; Clear the modified mark if appropriate
	  (bdiff)))))

(defun insert-menu-bar ()
  (interactive)
  (while (not (looking-back-at "\n\n"))
    (insert "\n"))
  (insert "<hr>\n[ ")
  ;; Next, Previous
  (let ((prev-and-next (menu-bar-dir-lookup buffer-file-name)))
    ;; Next
    (if (cdr prev-and-next)
	(insert "<a href=\"" (cdr prev-and-next) "\">Next</a>" " | \n"))
    ;; Previous
    (if (car prev-and-next)
	(insert "<a href=\"" (car prev-and-next) "\">Previous</a>" " | \n")))
  (let ((index-file-p (string= (file-name-directory buffer-file-name)
			     "/home/fair/WWW/index/"))
	(up-url (if (and buffer-file-name
			 (member (file-name-nondirectory buffer-file-name)
				 '("default.html" "homepage.html"
				   "index.html" "toc.html")))
		    "../" "./"))
	(top-url (top-directory)))
    ;; Up
    ;; Checking index-file-p doesn't hurt index/index.html because for it,
    ;; up-url = top-url anyway.
    (if (not (or index-file-p
		 (string= up-url top-url)
		 (and (string= up-url "./")
		      (string= top-url ""))
		 ))
	(progn
	  (insert "<a href=\"" up-url "\">Up</a>")
	  (insert " | \n")))
    ;; Top
    (insert "<a href=\"" (top-directory) "\">FAIR</a>")
    (insert " | \n")
    ;; Comments
    (insert "<a href=\"http://www.ai.mit.edu/people/mernst/fair/send-comment-www.html\">Mail/Suggest</a>")
    ;; Index
    (if (not index-file-p)
	(insert " | \n" "<a href=\"" (top-directory) "index/\">Index</a>"))
    (insert " ]<br>\n")))

(defun update-menu-bars (directory)
  (interactive "DUpdate HTML menu bars for directory: ")
  (read-menu-bar-structure)
  (let* ((default-directory directory)
	 (files (directory-files default-directory t "\\.html$")))
    (while files
      (if (and (not (equal (car files) "/home/fair/WWW/index.html"))
	       (not (stringp (car (file-attributes (car files))))))
	  ;; not the top-level homepage, and not a symbolic link
	  (progn
	    (find-file (car files))
	    (update-menu-bar)))
      (setq files (cdr files)))))

(defun insert-menu-bars (directory)
  (interactive "DInsert HTML menu bars for directory: ")
  (read-menu-bar-structure)
  (let* ((default-directory directory)
	 (files (directory-files default-directory nil "\\.html$")))
    (while files
      (find-file (car files))
      (if (not (equal buffer-file-name "/u/mernst/WWW/fair/homepage.html"))
	  (save-excursion
	    (goto-char (point-max))
	    (insert "\n\n")
	    (insert-menu-bar)))
      (setq files (cdr files)))))

(defvar menu-bar-structure-alist '())
(defvar menu-bar-file "~fair/WWW/menu-bar-structure")
(defvar menu-bar-file-time nil)

(defun read-menu-bar-structure (&optional force)
  (if (or force
	  (not menu-bar-file-time)
	  (let ((disk-file-time (nth 5 (file-attributes menu-bar-file))))
	    (prog1
		(file-time-< menu-bar-file-time disk-file-time)
	      (setq menu-bar-file-time disk-file-time))))
      (save-window-excursion
	(set-buffer (get-buffer-create "*menu-bar-structure*"))
	(erase-buffer)
	(insert-file-contents menu-bar-file)
	(setq menu-bar-structure-alist
	      ;; This should be the identity function; but just in case
	      (mapcar (function (lambda (pair)
				  (cons (file-name-as-directory (car pair))
					(cdr pair))))
		      (read (current-buffer))))
	(set-buffer-modified-p nil)
	(kill-buffer (current-buffer)))))

;; Return a cons of the base filename (which may or may not appear in the list)
;; and the 
(defun menu-bar-dir-lookup (file)
  (let ((dir-assoc (assoc (file-name-directory file)
			  menu-bar-structure-alist)))
    (or (and dir-assoc
	     (elt+list->prev-and-next (file-name-nondirectory file)
				      (cdr dir-assoc)))
	;; Try going one level (but no more) deeper
	(let ((dir-assoc (assoc (file-name-directory
				 (directory-file-name
				  (file-name-directory file)))
				menu-bar-structure-alist)))
	  (if dir-assoc
	      (let ((result (elt+list->prev-and-next
			     (concat (file-name-nondirectory
				      (directory-file-name
				       (file-name-directory file)))
				     "/" (file-name-nondirectory file))
			     (cdr dir-assoc))))
		(and result
		     (cons (and (car result) (concat "../" (car result)))
			   (and (cdr result) (concat "../" (cdr result)))))))))))
;; (menu-bar-dir-lookup "/u/mernst/WWW/fair/9412/homepage.html")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; HTML created by Word Internet Assistant
;;;

(defun touchup-internet-assistant ()
  (interactive)
  (goto-char (point-min))
  (while (search-forward "---" nil t)
    (backward-delete-char 1)
    (if (looking-at "\\b")
	(progn
	  (insert " ")
	  (backward-char 1)))
    (backward-char 2)
    (if (looking-back-at "\\b")
	(insert " ")))
  (replace-string "---" "--")
  (goto-char (point-min))
  (replace-string "&quot;" "\"")
  (goto-char (point-min))
  (while (re-search-forward "[ \t\n]\"[ \t\n]" nil t)
    (backward-char 2)
    (message "Extra space at Beginning or End? (b or n)")
    (let ((done nil))
      (while (not done)
	(let ((char (read-char)))
	  (cond ((member char '(?b ?B))
		 (backward-delete-char 1)
		 (backward-char 1)
		 (setq done t))
		((member char '(?e ?E))
		 (forward-char 1)
		 (delete-char 1)
		 (backward-char 2)
		 (setq done t))
		(t
		 (beep)))))))
  (goto-char (point-min))
  (while (re-search-forward "<h[1-5]" nil t)
    (if (not (= ?\n (char-after (- (match-beginning 0) 2))))
	(progn
	  (backward-char 3)
	  (insert ?\n)
	  (forward-char 3)))
    (let ((first-num-point (1- (point))))
      (search-forward "</h")
      (if (y-or-n-p (concat "Demote \""
			    (buffer-substring (+ 2 first-num-point)
					      (- (point) 3))
			    "\"? "))
	  (let ((new-level-char (1+ (char-after (point)))))
	    (delete-char 1)
	    (insert new-level-char)
	    (goto-char first-num-point)
	    (delete-char 1)
	    (insert new-level-char)))))
  (goto-char (point-min))
  (while (re-search-forward "</menu>\n+<ul>" nil t)
    (delete-region (1- (match-beginning 0)) (match-end 0))
    (search-backward "<menu>")
    (forward-char 1)
    (delete-char 4)
    (insert "UL"))
  (message "Done touching up."))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Utilities
;;;

(defun file-time-< (ft1 ft2)
  (or (< (car ft1) (car ft2))
      (and (= (car ft1) (car ft2))
	   (< (car (cdr ft1)) (car (cdr ft2))))))

;; Return a cons of the elements before and after ELT in LIST.
;; Either of the cons components may be nil.
;; Nil return value means ELT isn't in LIST>
(defun elt+list->prev-and-next (elt list)
  (let ((prev nil)
	(this list))
    (while (and this (not (equal elt (car this))))
      (setq prev this
	    this (cdr this)))
    (and this
	 (cons (car prev) (car (cdr this))))))
;; (elt+list->prev-and-next '4 '(1 2 3 4 5 6 7 8 9))
;; (elt+list->prev-and-next '1 '(1 2 3 4 5 6 7 8 9))
;; (elt+list->prev-and-next '9 '(1 2 3 4 5 6 7 8 9))
;; (elt+list->prev-and-next '0 '(1 2 3 4 5 6 7 8 9))


;; Not needed any more -- all done.
(defun convert-from-<index> ()
  (interactive)
  (save-excursion
    (let ((directory-part (if (equal (file-name-directory (buffer-file-name))
				     "/u/mernst/WWW/fair/")
			      ""
			    "../")))
      (goto-char (point-min))
      (while (re-search-forward "<index roman=\"\\([^\"]*\\)\">" nil t)
	(replace-match (concat "<a href=\"" directory-part "index.html#\\1\">"))
	(search-forward "</index>")
	(replace-match "</a>"))
      (goto-char (point-min))
      (while (re-search-forward "<index quoted=\"\\([^\"]*\\)\">" nil t)
	(replace-match
	 (concat "<a href=\"" directory-part "index.html#\\1|q\">"))
	(search-forward "</index>")
	(replace-match "</a>"))
      (goto-char (point-min))
      (while (re-search-forward "<index italic=\"\\([^\"]*\\)\">" nil t)
	(replace-match
	 (concat "<a href=\"" directory-part "index.html#\\1|i\">"))
	(search-forward "</index>")
	(replace-match "</a>")))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; HTTP usage information
;;;

;; 4/3/95: 1180, 2386
;; 3/27/95: 1388, 5045
;; 3/20/95: 1706, 5796
;; 3/13/95: 1417, 4618
;; 3/6/95: 1378, 4900
;; 2/27/95: 1127, 4448
;; 2/20/95: 873, 3709
;; 2/13/95: 774, 3417
;; 2/6/95: 631, 2650
;; 1/30/95: 591, 2652
;; 1/23/95: 410, 1740
;; 1/16/95: 384, 1644
;; 1/9/95: 414, 1978

;; This is for FAIR only, not media watch:
;; Call this from a file like /o/log/httpd/access_log.Feb.13.1995.Z
(defun fair-http-usage ()
  (interactive)
  (httpd-log-trim-to-fair)
  (let ((buffer-read-only nil))
    (let ((total-accesses (count-lines (point-min) (point-max))))
      (sort-lines nil (point-min) (point-max))
      (while (search-forward " - - " nil t)
	(delete-region (point) (progn (end-of-line 1) (point))))
      (shell-command-on-region (point-min) (point-max) "uniq" t)
      (message "Total accesses: %d   unique machines: %d"
	       total-accesses (count-lines (point-min) (point-max))))))

(defun fair-http-usage-detailed ()
  (interactive)
  (httpd-log-trim-to-fair)
  (let ((buffer-read-only nil))
    (let ((total-accesses (count-lines (point-min) (point-max))))
      (sort-lines nil (point-min) (point-max))
      (while (search-forward " - - " nil t)
	(delete-region (point) (progn (end-of-line 1) (point))))
      (shell-command-on-region (point-min) (point-max) "uniq" t)
      (message "Total accesses: %d   unique machines: %d"
	       total-accesses (count-lines (point-min) (point-max))))))

(defun httpd-log-trim-to-fair ()
  (let ((buffer-read-only nil))
    (buffer-disable-undo (current-buffer))
    (goto-char (point-min))
    (delete-non-matching-lines "mernst/fair")
    (goto-char (point-min))
    (delete-matching-lines "monozygote\\.research\\.microsoft\\.com")
    (goto-char (point-min))
    (delete-matching-lines "index\\([-/].\\)\\.html")
    (goto-char (point-min))
    (delete-matching-lines "\\.gif")
    (goto-char (point-min))
    (while (search-forward "\000" nil t)
      (backward-delete-char 1)
      (if (looking-at "\000+")
	  (delete-region (match-beginning 0) (match-end 0))))
    (goto-char (point-min))
    (while (search-forward "%7E" nil t)
      (replace-match "~"))))

;; After this, use "sort | uniq -c".
(defun httpd-log-trim-to-url ()
  (let ((buffer-read-only nil))
    (buffer-disable-undo (current-buffer))
    (goto-char (point-min))
    (while (re-search-forward "\"\\(GET\\|HEAD\\) " nil t)
      (delete-region (point) (progn (beginning-of-line) (point)))
      (re-search-forward " ")
      (backward-char 1)
      (if (= ?\" (preceding-char))
	  (backward-char 1))
      (if (= ?\/ (preceding-char))
	  (backward-char 1))
      (delete-region (point) (progn (end-of-line) (point))))))

(defun fair-make-tags ()
  (interactive)
  (let ((default-directory "/u/mernst/WWW/fair/"))
    (shell-command
     "etags *.html 9311/*.html 9401/*.html 9403/*.html 9405/*.html 9407/*.html 9409/*.html 9411/*.html 9412/*.html 9501/*.html best-of-extra/*.html counterspin/*.html media-beat/*.html")))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Shorten some lines of the hardcopy index
;;;

(defun shorten-hardcopy-index-lines ()
  (interactive)
  (goto-char (point-min))
  (while (not (eobp))
    (end-of-line)
    (if (> (current-column) 85)
	(let ((last-char (current-column)))
	  (backward-sexp 1)
	  (backward-delete-char (- last-char 81))
	  (insert "... ")))
    (forward-line 1)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Backup file management
;;;

;; Restore backup .html files
(defun bak-replace-maybe ()
  (interactive)
  (save-excursion
    (let ((filename (buffer-substring (point) (progn (end-of-line) (point)))))
      (diff filename (concat filename ".bak"))
      (sit-for 1)
      (set-buffer "*compilation*")
      (while (not (progn
		    (goto-char (point-max))
		    (forward-line -1)
		    (looking-at "diff exited")))
	(sit-for 1))
      (if (or (not (= 8 (count-lines (point-min) (point-max))))
	      (not (string= "> <title>"
			    (buffer-substring (progn (goto-char (point-max)) (forward-line -5) (point))
					      (+ (point) 9)))))
	  (error "maybe a real difference"))
      (rename-file (concat filename ".bak") filename t)
      (message "Replacement done."))))
